﻿Imports System.IO
Imports System.Xml.Linq
Imports System.Linq


''' <summary>
''' The PlayList class organizes a series of Albums by Shelf, and position on that Shelf.
''' </summary>
''' <remarks>This class inherits from SortedList, and strongly types the primary element to store as an Album object.</remarks>
Public Class PlayList
    Inherits SortedList(Of Integer, Album)

    Private _Name As String
    Private _File As String
    Private _MusicBasePath As String

    ''' <summary>
    ''' The default constructor
    ''' </summary>
    ''' <param name="PlayListName">The new PlayList name</param>
    ''' <param name="PlayListFileName">The file name of the playlist</param>
    ''' <remarks>This sets the internal properties for the play list so that the Load method(s) can function.</remarks>
    Public Sub New(ByVal PlayListName As String, _
                   ByVal PlayListFileName As String)
        ' TODO: Throw an exception if one or more of the properties isn't set
        Me._Name = PlayListName
        Me._File = Path.Combine(WPFMediaPlayer.AppSettings.Core.MetaDataPath, PlayListFileName)
        Me._MusicBasePath = WPFMediaPlayer.AppSettings.Core.MusicBasePath
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This checks for the existence of the playlist XML file.
    ''' </summary>
    ''' <value>Boolean</value>
    ''' <returns>This returns True if the file exists, and False if not.</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Exists()
        Get
            Return File.Exists(Me._File)
        End Get
    End Property


    ''' <summary>
    ''' This exposes the playlist Name.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Name()
        Get
            Return Me._Name
        End Get
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This creates the meta data for a blank playlist, and overwrites any existing one by the same name.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub CreateBlankPlayListMetaData()
        Dim BlankMediaPlayList As XDocument

        ' TODO: Consider making this a constant
        BlankMediaPlayList = _
                <?xml version="1.0" encoding="UTF-16" standalone="yes"?>
                <playlist>
                </playlist>

        BlankMediaPlayList.Save(Me._File)
    End Sub


    ''' <summary>
    ''' This creates a playlist that contains all of the media, and overwrites any existing one by the same name.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub CreateFullPlayListMetaData()
        Dim AllMediaPlayList As XDocument
        Dim AlbumsXML, CurrentAlbumXML, SongsXML, CurrentSongXML As XElement
        Dim ArtistFolders(), CurrentArtistFolder, CurrentArtistName As String
        Dim AlbumFolders(), CurrentAlbumFolder, CurrentAlbumName As String
        Dim SongFiles(), CurrentSongFile, CurrentSongName As String
        Dim AlbumIndex, SongIndex As Integer
        Dim HasMusic As Boolean


        ' TODO: Throw an exception if one or more of the properties isn't set

        ArtistFolders = System.IO.Directory.GetDirectories(Me._MusicBasePath)
        AlbumIndex = 0
        AlbumsXML = <albums></albums>
        For Each CurrentArtistFolder In ArtistFolders
            CurrentArtistName = CurrentArtistFolder.Substring(CurrentArtistFolder.LastIndexOf(System.IO.Path.DirectorySeparatorChar) + 1)

            AlbumFolders = System.IO.Directory.GetDirectories(CurrentArtistFolder)
            For Each CurrentAlbumFolder In AlbumFolders
                HasMusic = False
                CurrentAlbumName = CurrentAlbumFolder.Substring(CurrentAlbumFolder.LastIndexOf(System.IO.Path.DirectorySeparatorChar) + 1)

                CurrentAlbumXML = <album>
                                      <name><%= CurrentAlbumName %></name>
                                      <artist><%= CurrentArtistName %></artist>
                                      <jacketart><%= System.IO.Path.Combine(CurrentAlbumFolder, "AlbumArtSmall.jpg") %></jacketart>
                                      <position><%= AlbumIndex %></position>
                                  </album>

                ' TODO: Don't limit to WMA files
                SongsXML = <songs></songs>
                SongFiles = Directory.GetFiles(CurrentAlbumFolder, "*.wma")
                SongIndex = 0
                For Each CurrentSongFile In SongFiles
                    If (FileLen(CurrentSongFile) > 0) Then
                        CurrentSongName = Path.GetFileNameWithoutExtension(CurrentSongFile)
                        CurrentSongXML = <song>
                                             <name><%= CurrentSongName %></name>
                                             <pathtomedia><%= CurrentSongFile %></pathtomedia>
                                             <position><%= SongIndex %></position>
                                         </song>
                        SongsXML.Add(CurrentSongXML)
                        SongIndex += 1
                        HasMusic = True
                    End If
                Next

                If (HasMusic) Then
                    CurrentAlbumXML.Add(SongsXML)
                    AlbumsXML.Add(CurrentAlbumXML)
                    AlbumIndex += 1
                End If
            Next
        Next


        ' TODO: Consider making this a constant
        AllMediaPlayList = _
                <?xml version="1.0" encoding="UTF-16" standalone="yes"?>
                <playlist>
                    <%= From CurrentAlbum In AlbumsXML.<album> Select CurrentAlbum %>
                </playlist>

        AllMediaPlayList.Save(Me._File)
    End Sub


    ''' <summary>
    ''' This will load the playlist's contents
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Load()
        Dim CurrentPlayList As XDocument
        Dim CurrentAlbum, CurrentSong As XElement
        Dim CurrentArtistName As String
        Dim NewAlbum As Album
        Dim HasMusic As Boolean

        CurrentPlayList = XDocument.Load(Me._File)
        For Each CurrentAlbum In (From A In CurrentPlayList...<album> Select A)
            CurrentArtistName = CurrentAlbum.<artist>.Value

            ' Load up the album information
            NewAlbum = New Album(CurrentAlbum.<name>.Value, _
                                 CurrentAlbum.<artist>.Value, _
                                 CurrentAlbum.<jacketart>.Value, _
                                 CurrentAlbum.<position>.Value)

            ' Load up the songs for the album
            HasMusic = False
            For Each CurrentSong In (From S In CurrentAlbum...<song> Select S)
                HasMusic = True
                NewAlbum.Songs.Add(New Song(CurrentSong.<name>.Value, _
                                            CurrentSong.<pathtomedia>.Value, _
                                            CurrentSong.<position>.Value))
            Next

            ' Add the album to the collection if there was at least one song attached to it
            If (HasMusic) Then Me.Add(CurrentAlbum.<position>.Value, NewAlbum)
        Next
    End Sub


    ''' <summary>
    ''' This saves the playlist back to disk.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Save()
        Dim AllMediaPlayList As XDocument

        AllMediaPlayList = _
                <?xml version="1.0" encoding="UTF-16" standalone="yes"?>
                <playlist>
                    <%= From CurrentAlbum In Me Select <album>
                                                           <name><%= CurrentAlbum.Value.Name %></name>
                                                           <artist><%= CurrentAlbum.Value.Artist %></artist>
                                                           <jacketart><%= CurrentAlbum.Value.JacketArtPath %></jacketart>
                                                           <position><%= CurrentAlbum.Value.Index %></position>
                                                           <songs>
                                                               <%= From CurrentSong In CurrentAlbum.Value.Songs Select <song>
                                                                                                                           <name><%= CurrentSong.Name %></name>
                                                                                                                           <pathtomedia><%= CurrentSong.PathToMedia %></pathtomedia>
                                                                                                                           <position><%= CurrentSong.Index %></position>
                                                                                                                       </song> %>
                                                           </songs>
                                                       </album> %>
                </playlist>

        AllMediaPlayList.Save(Me._File)
    End Sub
#End Region


End Class
