﻿Imports System.IO
Imports System.Xml.Linq
Imports System.Linq

''' <summary>
''' This class encapsulates the logic for managing the top-level meta data for the player.
''' </summary>
''' <remarks></remarks>
Public Class PlayerMetaData

    Private _MetaDataPath As String
    Private _MetaDataMasterFileName As String
    Private _MusicBasePath As String
    Private _PlayLists As List(Of PlayList)


    ' TODO: Consider moving these to the config file
    Private Const ALL_MEDIA_PLAYLIST_DISPLAY_NAME As String = "All Media"
    Private Const ALL_MEDIA_PLAYLIST_FILENAME As String = "AllMedia.PlayList.xml"


#Region " Custom Exceptions "
    ''' <summary>
    ''' This is used when the user tries to create a playlist with the same name as one that already exists.
    ''' </summary>
    ''' <remarks></remarks>
    Public Class PlaylistAlreadyExistsException
        Inherits System.Exception
    End Class

    ''' <summary>
    ''' This is used when the user tries to delete the "default" playlist, defined
    ''' by PlayerMetaData.ALL_MEDIA_PLAYLIST_DISPLAY_NAME
    ''' </summary>
    ''' <remarks></remarks>
    Public Class DefaultPlayListCannotBeDeletedException
        Inherits System.Exception
    End Class
#End Region


    ''' <summary>
    ''' The main constructor
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        Me._MetaDataPath = WPFMediaPlayer.AppSettings.Core.MetaDataPath
        Me._MetaDataMasterFileName = WPFMediaPlayer.AppSettings.Core.MetaDataMasterFileName
        Me._MusicBasePath = WPFMediaPlayer.AppSettings.Core.MusicBasePath

        Me.ReloadAllPlaylists(False)
    End Sub


#Region " Custom Exception Classes "
    Public Class CeaseLoadingException
        Inherits System.Exception
    End Class
#End Region


#Region " Properties "
    ''' <summary>
    ''' This checks to see if the top-level meta data exists.
    ''' </summary>
    ''' <value>Boolean</value>
    ''' <returns>This returns True if the meta data is found, and false otherwise.</returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Exists()
        Get
            Return File.Exists(Path.Combine(Me._MetaDataPath, Me._MetaDataMasterFileName))
        End Get
    End Property


    ''' <summary>
    ''' This returns the full file and path to the master meta data file
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property MetaDataFileFullPath() As String
        Get
            Return Path.Combine(Me._MetaDataPath, Me._MetaDataMasterFileName)
        End Get
    End Property


    ''' <summary>
    ''' This exposes the playlists list, as loaded from the meta data
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property PlayLists() As List(Of PlayList)
        Get
            Return Me._PlayLists
        End Get
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This creates the top-level meta data.
    ''' </summary>
    ''' <param name="ShouldCreateDefaultPlayList">This indicates whether or not the default playlist should be forcibly created.</param>
    ''' <remarks></remarks>
    Public Sub Initialize(ByVal ShouldCreateDefaultPlayList As Boolean)
        If (Not Directory.Exists(Me._MetaDataPath)) Then Directory.CreateDirectory(Me._MetaDataPath)

        If (Not File.Exists(Me.MetaDataFileFullPath)) Then
            Dim NewPlayListsFileContents As XDocument = _
                <?xml version="1.0" encoding="UTF-16" standalone="yes"?>
                <playlists>
                    <playlist>
                        <name><%= ALL_MEDIA_PLAYLIST_DISPLAY_NAME %></name>
                        <filename><%= ALL_MEDIA_PLAYLIST_FILENAME %></filename>
                    </playlist>
                </playlists>

            NewPlayListsFileContents.Save(Me.MetaDataFileFullPath)
        End If

        Me.ReloadAllPlaylists(ShouldCreateDefaultPlayList)
    End Sub



    ''' <summary>
    ''' This loads up the playlists
    ''' </summary>
    ''' <param name="ShouldCreateDefaultPlayList">This indicates whether or not the default playlist should be forcibly created.</param>
    ''' <remarks></remarks>
    Public Sub ReloadAllPlaylists(ByVal ShouldCreateDefaultPlayList As Boolean)
        Dim MasterPlayListsRaw As XDocument
        Dim CurrentPlayListXML As XElement
        Dim NewPlayList As PlayList
        Dim Answer As MsgBoxResult

        ' Return if the path doesn't exist yet.  The calling code needs to call Initialize first.
        If (Not Me.Exists) Then Return

        ' Load up the play lists defined in the master list
        MasterPlayListsRaw = New XDocument
        MasterPlayListsRaw = XDocument.Load(Me.MetaDataFileFullPath)
        Dim PlayListsRaw = From playlist In MasterPlayListsRaw...<playlist> _
                           Select playlist


        ' Load up the play lists
        Me._PlayLists = New List(Of PlayList)
        For Each CurrentPlayListXML In PlayListsRaw
            NewPlayList = New PlayList(CurrentPlayListXML.<name>.Value, CurrentPlayListXML.<filename>.Value)
            If (Not NewPlayList.Exists) Then
                If (ShouldCreateDefaultPlayList) Then
                    Answer = MsgBoxResult.Yes
                    ShouldCreateDefaultPlayList = False
                Else
                    Answer = MsgBox("The playlist '" & CurrentPlayListXML.<name>.Value & "' was not found, or you don't have permission to it.  Would you like to: " & vbCrLf & vbCrLf & "1) Create it, and assign all media to it (Yes)" & vbCrLf & "2) Create it, but empty (No)" & vbCrLf & "3) Or exit (Cancel)", MsgBoxStyle.YesNoCancel)
                End If

                ' Handle the user's response to what kind of playlist should be created.
                Select Case Answer
                    Case MsgBoxResult.Yes
                        NewPlayList.CreateFullPlayListMetaData()

                    Case MsgBoxResult.No
                        NewPlayList.CreateBlankPlayListMetaData()

                    Case Else
                        Throw New CeaseLoadingException
                End Select
            End If

            ' The playlist meta data exists now for the playlist, so load the playlist itself
            NewPlayList.Load()

            Me._PlayLists.Add(NewPlayList)
        Next
    End Sub


    ''' <summary>
    ''' This updates the master meta data file, creates the blank playlist file, and updates the internal structures.
    ''' </summary>
    ''' <param name="NewPlayListName"></param>
    ''' <remarks></remarks>
    Public Sub AddNewPlayList(ByVal NewPlayListName As String)
        Dim MasterPlayListsDocContents As XDocument
        Dim NewPlayListElementContents As XElement
        Dim NewPlayList As PlayList
        Dim NewPlayListFileName, NewPlayListFullPath As String

        NewPlayListFileName = NewPlayListName & ".PlayList.xml"
        NewPlayListFullPath = Path.Combine(Me._MetaDataPath, NewPlayListFileName)
        If (File.Exists(NewPlayListFullPath)) Then Throw New PlaylistAlreadyExistsException
        ' TODO: The above only looks for the existence of the .PlayList.xml file.  It doesn't fail if
        ' the file doesn't exist, but the name still exists in the Playlists.xml file.  This situation,
        ' however will be covered when the player boots up.  There is still the possibility for a
        ' problem if the user boots the player up, then deletes the .PlayList.xml file, and then tries
        ' to create a new one with the same name.

        ' Update the Master Meta Data File
        MasterPlayListsDocContents = New XDocument
        MasterPlayListsDocContents = XDocument.Load(Me.MetaDataFileFullPath)

        NewPlayListElementContents = <playlist>
                                         <name><%= NewPlayListName %></name>
                                         <filename><%= NewPlayListFileName %></filename>
                                     </playlist>

        MasterPlayListsDocContents.Descendants...<playlist>.Last.AddAfterSelf(NewPlayListElementContents)
        MasterPlayListsDocContents.Save(Me.MetaDataFileFullPath)

        ' Add this new playlist to the collection, and load it
        NewPlayList = New PlayList(NewPlayListName, NewPlayListFileName)
        NewPlayList.CreateBlankPlayListMetaData()
        NewPlayList.Load()
        Me._PlayLists.Add(NewPlayList)

    End Sub

    ''' <summary>
    ''' This removes the playlist from the meta data file, removes the .PlayList.xml file, and updates the internal structure.
    ''' </summary>
    ''' <param name="PlayListName">The name of the playlist to remove.</param>
    ''' <returns></returns>
    ''' <remarks>This method will remove the first playlist with the name supplied.</remarks>
    Public Function DeletePlayList(ByVal PlayListName As String)
        Dim MasterPlayListsDocContents As XDocument
        Dim CurrentPlayList As XElement
        Dim CurrentPlayListPath As String
        Dim PlayListToDeleteIndex As Integer

        If (PlayListName = ALL_MEDIA_PLAYLIST_DISPLAY_NAME) Then Throw New DefaultPlayListCannotBeDeletedException

        MasterPlayListsDocContents = New XDocument
        MasterPlayListsDocContents = XDocument.Load(Me.MetaDataFileFullPath)

        ' Find it in the meta data file, and remove it
        CurrentPlayListPath = ""
        PlayListToDeleteIndex = 0
        For Each CurrentPlayList In MasterPlayListsDocContents.Descendants...<playlist>
            If (CurrentPlayList.<name>.Value = PlayListName) Then
                CurrentPlayListPath = CurrentPlayList.<filename>.Value
                CurrentPlayList.Remove()
                Exit For
            End If
            PlayListToDeleteIndex += 1
        Next
        MasterPlayListsDocContents.Save(Me.MetaDataFileFullPath)

        ' Remove the corresponding .PlayList.xml file
        If (CurrentPlayListPath.Trim.Length = 0) Then Return False
        If (Not File.Exists(Path.Combine(Me._MetaDataPath, CurrentPlayListPath))) Then Return False
        File.Delete(Path.Combine(Me._MetaDataPath, CurrentPlayListPath))

        ' Update the internal structure.  The position within the _PlayLists collection is the same
        ' as the position within the meta data file (as determined above)
        Me._PlayLists.RemoveAt(PlayListToDeleteIndex)

        Return True
    End Function
#End Region





End Class
