﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation
Imports WPFMediaPlayer.BusinessObjects

''' <summary>
''' This encapsulates the logic and properties for a JewelCase, the UI representation of the WPFMediaPlayer.BusinessObjects.Album class.
''' </summary>
''' <remarks></remarks>
Partial Public Class JewelCase
    Inherits System.Windows.Controls.UserControl

    Private _CurrentAlbum As Album
    Private _LeftMostCoordinate As Double


    ''' <summary>
    ''' This initializes a single JewelCase object, based on the Album object passed in.
    ''' </summary>
    ''' <param name="CurrentAlbum">The Album to represent as a JewelCase</param>
    ''' <param name="NewIndex">The JewelCase's index, as it sits on the Shelf</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal CurrentAlbum As Album, ByVal NewIndex As Integer)
        MyBase.New()

        Me.InitializeComponent()

        Me.MainImage.Source = Me.GetJacketArtSource(CurrentAlbum.JacketArtPath)
        Me._CurrentAlbum = CurrentAlbum
        ' Count up the pixels taken up by all of the albums to the left, add the current one, but subtract
        ' off the current JewelCase's right margin so that you can drop another JewelCase in between
        ' two others, and it will be recognized as being before the second one.  If you leave the
        ' final right margin measurement in the equation, dropping a JewelCase between two others
        ' will mean that it will be placed before the first one.
        Me._LeftMostCoordinate = (NewIndex * (Me.Width + Me.Margin.Left + Me.Margin.Right)) - Me.Margin.Right
        Me.ToolTip = CurrentAlbum.Name & vbCrLf & CurrentAlbum.Artist
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This gets the index of the JewelCase as it sits on the Shelf, 0-indexed.  It is also used to set it.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Index() As Integer
        Get
            Return Me._CurrentAlbum.Index
        End Get
        Set(ByVal value As Integer)
            Me._CurrentAlbum.Index = value
        End Set
    End Property


    ''' <summary>
    ''' This returns the jacket art used as the main image.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property JacketArt() As BitmapImage
        Get
            Return Me.MainImage.Source
        End Get
    End Property


    ''' <summary>
    ''' This returns the left-most coordinate of the current JewelCase.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property LeftMostCoordinate() As Double
        Get
            Return Me._LeftMostCoordinate
        End Get
    End Property


    ''' <summary>
    ''' This returns the Album object that this JewelCase is presenting.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property SourceAlbum() As Album
        Get
            Return Me._CurrentAlbum
        End Get
    End Property
#End Region


#Region " Events "
    Public Event JewelCaseOpened(ByVal sender As Album, ByVal e As EventArgs)
    Public Event Spotlighted(ByVal sender As JewelCase, ByVal e As JewelCaseHighlightArgs)
    Public Event UnSpotlighted(ByVal sender As JewelCase)
#End Region


#Region " Methods "
    ''' <summary>
    ''' This function loads up the image to display on the jewelcase from a file, and returns
    ''' a BitmapImage object, which can then be assigned to the Source property of the Image
    ''' object in the JewelCase container.
    ''' </summary>
    ''' <param name="JacketArtImagePath">The full path and file name of the image to load up.</param>
    ''' <returns>BitmapImage</returns>
    ''' <remarks></remarks>
    Private Function GetJacketArtSource(ByVal JacketArtImagePath As String) As BitmapImage
        If (Not File.Exists(JacketArtImagePath)) Then JacketArtImagePath = "MissingJacketArt.jpg"

        ' Winforms Image we want to get the WPF Image from... 
        Dim MySourceImage As System.Drawing.Image = System.Drawing.Image.FromFile(JacketArtImagePath)

        ' Source for the image...
        Dim MyBitmap As New BitmapImage
        MyBitmap.BeginInit()
        Dim MyMemoryStream As New MemoryStream

        ' Save to a memory stream...
        MySourceImage.Save(MyMemoryStream, System.Drawing.Imaging.ImageFormat.Bmp)

        ' Rewind the stream... 
        MyMemoryStream.Seek(0, SeekOrigin.Begin)

        ' Tell the WPF image to use this stream... 
        MyBitmap.StreamSource = MyMemoryStream
        MyBitmap.EndInit()

        ' And your WPF image is ready to go! 
        Return MyBitmap

        ' TODO: Good idea to close that stream once your image is loaded.  However, this
        ' next line closes the stream too early, and the code above doesn't have a chance
        ' to really load it.
        'MyMemoryStream.Close()
    End Function
#End Region


#Region " Event Handlers "
    ''' <summary>
    ''' This catches the MouseDoubleClick event, and raises the custom event "JewelCaseSelected", passing the
    ''' Album object that is tied to this JewelCase with it.
    ''' </summary>
    ''' <param name="sender">The Album object that was used to create this instance of JewelCase</param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub JewelCase_MouseDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles Me.MouseDoubleClick
        RaiseEvent JewelCaseOpened(Me._CurrentAlbum, Nothing)
    End Sub


    ''' <summary>
    ''' This will determine where the bottom left corner of the current jewelcase is, and throws the Spotlighted
    ''' event, thus letting the handling code know which JewelCase raised the MouseEnter event, and where it is
    ''' located relative to the main window's upper left corner.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub JewelCase_MouseEnter(ByVal sender As Object, ByVal e As System.Windows.Input.MouseEventArgs) Handles Me.MouseEnter
        Dim FlyoutPosition As Point

        FlyoutPosition = Globals.OffsetFromMainWindow(Me.MainImage)
        FlyoutPosition.Y += Me.Height

        RaiseEvent Spotlighted(Me, New JewelCaseHighlightArgs(FlyoutPosition))
    End Sub


    ''' <summary>
    ''' This throws the UnSpotlighted event, letting the handling code know which JewelCase raised the event.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub JewelCase_MouseLeave(ByVal sender As Object, ByVal e As System.Windows.Input.MouseEventArgs) Handles Me.MouseLeave
        RaiseEvent UnSpotlighted(Me)
    End Sub

#End Region

End Class
