﻿Imports System
Imports System.IO
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Data
Imports System.Windows.Media
Imports System.Windows.Media.Animation
Imports System.Windows.Navigation
Imports WPFMediaPlayer.BusinessObjects

''' <summary>
''' This class encapsulates the business logic for a Shelf, an object that holds one or more Albums.
''' </summary>
''' <remarks></remarks>
Partial Public Class Shelf

    Private _Index As Integer
    Private _IsDragging As Boolean
    Private _JewelCaseStartPoint As Point


#Region " Events "
    Public Event AlbumOpened(ByVal CurrentAlbum As Album, ByVal e As System.EventArgs)
    Public Event AlbumSpotlighted(ByVal sender As JewelCase, ByVal e As JewelCaseHighlightArgs)
    Public Event AlbumUnSpotlighted(ByVal sender As JewelCase)
    Public Event JewelCaseMoved(ByVal sender As Shelf, ByVal SourceIndex As Integer, ByVal DestinationIndex As Integer)
#End Region


    ''' <summary>
    ''' The default constructor
    ''' </summary>
    ''' <param name="NewIndex">The new shelf's index within the parent container.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NewIndex As Integer)
        MyBase.New()

        Me.InitializeComponent()

        ' Insert code required on object creation below this point.
        Me._Index = NewIndex
        Me.AllowDrop = True
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This gets or sets the shelf's index.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Index() As Integer
        Get
            Return Me._Index
        End Get
        Set(ByVal value As Integer)
            Me._Index = value
        End Set
    End Property


    ''' <summary>
    ''' This exposes (or rather, aliases) the StackPanel contents.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property JewelCases() As UIElementCollection
        Get
            ' TODO: This should really return a strongly-typed collection of JewelCases.
            Return Me.LayoutRoot.Children
        End Get
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This adds the new album to the shelf, both to the internal collection, and visually.
    ''' </summary>
    ''' <param name="NewAlbum">The album to add</param>
    ''' <remarks></remarks>
    Public Sub AddAlbum(ByVal NewAlbum As Album)
        Dim NewJewelCase As JewelCase
        Dim NewJewelCaseIndex As Integer

        NewJewelCaseIndex = NewAlbum.Index
        NewJewelCase = New JewelCase(NewAlbum, NewJewelCaseIndex)

        AddHandler NewJewelCase.JewelCaseOpened, AddressOf OpenJewelCase
        AddHandler NewJewelCase.PreviewMouseLeftButtonDown, AddressOf RecordStartingPoint
        AddHandler NewJewelCase.Spotlighted, AddressOf HandleAlbumSpotlighted
        AddHandler NewJewelCase.UnSpotlighted, AddressOf HandleAlbumUnSpotlighted
        AddHandler NewJewelCase.PreviewMouseMove, AddressOf VerifyRealDrag

        Me.LayoutRoot.Children.Add(NewJewelCase)
    End Sub


    ''' <summary>
    ''' This performs the drag, and bundles up the Jewelcase index as the data being sent
    ''' </summary>
    ''' <param name="JewelCaseMoving">The JewelCase object that represents the drag source</param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub StartDrag(ByVal JewelCaseMoving As JewelCase, ByVal e As MouseEventArgs)
        Dim JewelCaseData As DataObject
        Dim JewelCaseEffects As DragDropEffects

        Me._IsDragging = True
        JewelCaseData = New DataObject(System.Windows.DataFormats.Text.ToString(), JewelCaseMoving.Index.ToString)
        JewelCaseEffects = DragDrop.DoDragDrop(JewelCaseMoving, JewelCaseData, DragDropEffects.Move)
        Me._IsDragging = False
    End Sub


    ''' <summary>
    ''' This retrieves the JewelCase that is to the left of the given X coordinate.
    ''' </summary>
    ''' <param name="TargetXCoordinate"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetJewelCaseToTheLeft(ByVal TargetXCoordinate As Double) As JewelCase
        Dim i As Integer
        Dim CurrentJewelCase As JewelCase

        ' TODO: This doesn't quite work when dragging something to the very end of the list. For now
        '       to drag something to the end of the list, drag it to the second to last position, and
        '       then drag the last JewelCase backwards one spot.
        ' TODO: This code seems to be a bit touchy.  It doesn't always seem to return the correct
        '       Jewelcase for the X coordinate passed in.
        For i = Me.LayoutRoot.Children.Count - 1 To 0 Step -1
            CurrentJewelCase = CType(Me.LayoutRoot.Children.Item(i), JewelCase)
            If (CurrentJewelCase.LeftMostCoordinate <= TargetXCoordinate) Then Return CurrentJewelCase
        Next

        Return Nothing
    End Function

#End Region


#Region " Event Handlers "
    ''' <summary>
    ''' This rethrows the JewelCase.JewelCaseOpened event as Shelf.AlbumOpened since the
    ''' _AlbumsOnShelf object, or the JewelCase objects on it are not exposed.
    ''' </summary>
    ''' <param name="AlbumSelected">The album selected</param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub OpenJewelCase(ByVal AlbumSelected As Album, ByVal e As System.EventArgs)
        RaiseEvent AlbumOpened(AlbumSelected, e)
    End Sub


    ''' <summary>
    ''' This records the initial Point for the jewel case selected
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>This is used by VerifyRealDrag to calculate the distance dragged.</remarks>
    Private Sub RecordStartingPoint(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
        Me._JewelCaseStartPoint = e.GetPosition(Nothing)
    End Sub


    ''' <summary>
    ''' This handles when a given jewelcase has been spotlighted (MouseEnter), and raises the AlbumSpotlighted event.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub HandleAlbumSpotlighted(ByVal sender As JewelCase, ByVal e As JewelCaseHighlightArgs)
        RaiseEvent AlbumSpotlighted(sender, e)
    End Sub


    ''' <summary>
    ''' This handles when a given jewelcase has been "unspotlighted" (MouseLeave), and raises the AlbumUnSpotlighted event.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <remarks></remarks>
    Private Sub HandleAlbumUnSpotlighted(ByVal sender As JewelCase)
        RaiseEvent AlbumUnSpotlighted(sender)
    End Sub


    ''' <summary>
    ''' This determines if the drag is more than the system minimums, and if so treats it as a real drag event.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub VerifyRealDrag(ByVal sender As Object, ByVal e As MouseEventArgs)
        Dim NewPosition As Point

        If (e.LeftButton = MouseButtonState.Pressed And Not Me._IsDragging) Then
            NewPosition = e.GetPosition(Nothing)

            If (Math.Abs(NewPosition.X - Me._JewelCaseStartPoint.X) > SystemParameters.MinimumHorizontalDragDistance Or _
                       Math.Abs(NewPosition.Y - Me._JewelCaseStartPoint.Y) > SystemParameters.MinimumVerticalDragDistance) Then
                Me.StartDrag(CType(sender, JewelCase), e)
            End If

        End If
    End Sub


    ''' <summary>
    ''' This process the drop for the Shelf, and positions the JewelCase being dragged into it's new home.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub Shelf_Drop(ByVal sender As Object, ByVal e As System.Windows.DragEventArgs) Handles Me.Drop
        Dim JewelCaseData As IDataObject = e.Data
        Dim IndexOfSourceJewelCase, IndexOfJewelCaseToInsertBefore, i As Integer
        Dim DragDestinationPoint As Point
        Dim JewelCaseMoving As JewelCase

        ' Make sure the right kind of data is being dropped here
        JewelCaseData = e.Data
        If (JewelCaseData.GetDataPresent(DataFormats.Text)) Then
            IndexOfSourceJewelCase = CType(JewelCaseData.GetData(DataFormats.Text), String)

            ' The position relative to the Shelf is relative to the upper left corner of the Shelf object,
            ' not merely what is visible on in the scrollviewer.
            DragDestinationPoint = e.GetPosition(Me)

            ' Move the JewelCase
            IndexOfJewelCaseToInsertBefore = Me.GetJewelCaseToTheLeft(DragDestinationPoint.X).Index
            JewelCaseMoving = CType(Me.LayoutRoot.Children.Item(IndexOfSourceJewelCase), JewelCase)
            Me.LayoutRoot.Children.Remove(JewelCaseMoving)
            Me.LayoutRoot.Children.Insert(IndexOfJewelCaseToInsertBefore, JewelCaseMoving)

            ' Walk the JewelCases on the Shelf, and rebuild their indexes
            For i = 0 To Me.LayoutRoot.Children.Count - 1
                CType(Me.LayoutRoot.Children.Item(i), JewelCase).Index = i
            Next

            ' Let the parent structure know that something changed
            RaiseEvent JewelCaseMoved(Me, IndexOfSourceJewelCase, IndexOfJewelCaseToInsertBefore)
        End If
    End Sub

#End Region

End Class
